// GameChuck
// Josh Villbrandt (http://javconcepts.com/)
// July 30, 2010

#include "pitches.h"
#define max_score 99 // decrease this for a shorter game (still just as hard... ^_^ )
#define initial_wait 1500 // increase this for an easier game
#define speaker_pin 8
#define num_buttons 6 // up to 6 (more if you add in the accelerometers!)l, ower this for an easier game
// this is particularly fun with num_buttons = 4

// Nunchuck vars
byte accx,accy,zbut,cbut;
int joyx, joyy;
boolean programOn = false;
long cbutLastDebounceTime = 0, zbutLastDebounceTime = 0, joyxLastDebounceTime = 0, joyyLastDebounceTime = 0;
long debounceDelay = 50;
byte cbutLastButtonState = 0, zbutLastButtonState = 0;
int joyxLastButtonState = 0, joyyLastButtonState = 0;
boolean cbutButtonFlipped = false, zbutButtonFlipped = false, joyxButtonFlipped = false, joyyButtonFlipped = false;

// Game Vars
int count = -1;
boolean new_round = true;
boolean game_over = false;
long round_start = 0;
int desired_button = -1; // 0=up,1=down,2=left,3=right,4=c,5=z
int last_desired_button = -1;
int desired_button_temp;
float wait_constant = max_score/log(initial_wait);

void setup() {
  // Start serial
  Serial.begin(19200);
  Serial.println("GameChuck");
  Serial.println("");
  Serial.println("Press the button as quick as you can!");
  
  // Seed random
  randomSeed(analogRead(0));
  
  // Start nunchuck
  //nunchuck_setpowerpins();
  nunchuck_init();
  
  // initialize lights
  sevenseg_spin(50, 8);
  update_leds(-1);
  
  // count down
  sevenseg(0, -1);
  Serial.println("3...");
  sevenseg(1, 3);
  tone(speaker_pin, NOTE_C4, 250);
  delay(1000);
  Serial.println("2...");
  sevenseg(1, 2);
  tone(speaker_pin, NOTE_C4, 250);
  delay(1000);
  Serial.println("1...");
  sevenseg(1, 1);
  tone(speaker_pin, NOTE_C4, 250);
  delay(1000);
}

void loop() {
  if(new_round) {
    // Stuff
    count = count + 1;
    new_round = false;
    sevenseg_score(count);
    
    // Pick random button
    last_desired_button = desired_button;
    desired_button_temp = random(num_buttons - 1); // prevents having the same thing twice in a row (otherwise would skip w/o debouncing)
    if(desired_button_temp >= desired_button) {
      if(desired_button_temp == (num_buttons - 1)) desired_button = 0;
      else desired_button = desired_button_temp + 1;
    }
    else desired_button = desired_button_temp;
    
    // Show user
    update_leds(desired_button);
    if(desired_button == 0) Serial.println("UP");
    else if(desired_button == 1) Serial.println("DOWN");
    else if(desired_button == 2) Serial.println("LEFT");
    else if(desired_button == 3) Serial.println("RIGHT");
    else if(desired_button == 4) Serial.println("C");
    else if(desired_button == 5) Serial.println("Z");
    round_start = millis();
  }
  
  // Update nunchuck data
  nunchuck_get_data();
  cbut = nunchuck_cbutton();
  zbut = nunchuck_zbutton();
  joyx = nunchuck_joyx();
  joyy = nunchuck_joyy();
  
  // Numbers based on stick position, fun!
  //sevenseg(0, (joyx-17)/17);
  //sevenseg(1, (joyy-17)/17);
  
  // Filter joystick inputs
  if(joyx > 192) joyx = 1;
  else if(joyx < 64) joyx = -1;
  else joyx = 0;
  if(joyy > 192) joyy = 1;
  else if(joyy < 64) joyy = -1;
  else joyy = 0;
  
  // Catch button changes
  if(cbut != cbutLastButtonState) {
      cbutLastDebounceTime = millis();
      cbutButtonFlipped = false;
  }
  if(zbut != zbutLastButtonState) {
      zbutLastDebounceTime = millis();
      zbutButtonFlipped = false;
  }
  if(joyx != joyxLastButtonState) {
      joyxLastDebounceTime = millis();
      joyxButtonFlipped = false;
  }
  if(joyy != joyyLastButtonState) {
      joyyLastDebounceTime = millis();
      joyyButtonFlipped = false;
  }
  
  // Debounce buttons
  if((millis() - cbutLastDebounceTime) > debounceDelay) {
    if(cbut == (byte)1 && !cbutButtonFlipped) {
      cbutButtonFlipped = true;
      
      // Action
    }
  }
  if((millis() - zbutLastDebounceTime) > debounceDelay) {
    if(zbut == (byte)1 && !zbutButtonFlipped) {
      zbutButtonFlipped = true;
      
      // Action
    }
  }
  if((millis() - joyxLastDebounceTime) > debounceDelay) {
    if(joyx > 0 && !joyxButtonFlipped) {
      joyxButtonFlipped = true;
      
      // Action
    }
    else if(joyx != 0 && !joyxButtonFlipped) {
      joyxButtonFlipped = true;
      
      // Action
    }
  }
  if((millis() - joyyLastDebounceTime) > debounceDelay) {
    if(joyy > 0 && !joyyButtonFlipped) {
      joyyButtonFlipped = true;
      
      // Action
    }
    else if(joyy != 0 && !joyyButtonFlipped) {
      joyyButtonFlipped = true;
      
      // Action
    }
  }
  
  // Save button states
  cbutLastButtonState = cbut;
  zbutLastButtonState = zbut;
  joyxLastButtonState = joyx;
  joyyLastButtonState = joyy;
  
  // Check buttons
  if(joyyButtonFlipped && joyy == 1 && last_desired_button != 0) {
    if(desired_button == 0) new_round = true;
    else game_over = true;
  }
  else if(joyyButtonFlipped && joyy == -1 && last_desired_button != 1) {
    if(desired_button == 1) new_round = true;
    else game_over = true;
  }
  else if(joyxButtonFlipped && joyx == -1 && last_desired_button != 2) {
    if(desired_button == 2) new_round = true;
    else game_over = true;
  }
  else if(joyxButtonFlipped && joyx == 1 && last_desired_button != 3) {
    if(desired_button == 3) new_round = true;
    else game_over = true;
  }
  else if(cbutButtonFlipped && last_desired_button != 4) {
    if(desired_button == 4) new_round = true;
    else game_over = true;
  }
  else if(zbutButtonFlipped && last_desired_button != 5) {
    if(desired_button == 5) new_round = true;
    else game_over = true;
  }
  
  // See if time has ellapsed for this round
  int wait = initial_wait - exp(count / wait_constant); // this make the game easy at first and much harder later
  if(!new_round && millis() > (round_start + wait)) game_over = true;
  
  // Game over?
  if(game_over) {
    tone(speaker_pin, NOTE_C4, 1000);
    Serial.println("Slow poke. You loose!");
    Serial.println("Your earned a score of " + count);
    while(true) {
      sevenseg(0, -1);
      sevenseg(1, -1);
      delay(500);
      sevenseg_score(count);
      delay(500);
    }
  }
  else if(new_round) tone(speaker_pin, NOTE_C4, 100);
}
